/*
 * This work is part of the White Rabbit Node Core project.
 *
 * Copyright (C) 2013-2014 CERN (www.cern.ch)
 * Author: Alessandro Rubini <rubini@gnudd.com>
 *
 * Released according to the GNU GPL, version 2 or any later version.
 */

#ifndef __TRTL_RT_PP_PRINTF_H
#define __TRTL_RT_PP_PRINTF_H

/**
 * @addtogroup trtl-rt-common
 * @{
 */

#include <stdarg.h>

#define CONFIG_PRINT_BUFSIZE 128


/**
 * It prints a string on the serial interface.
 * Internally, it uses the puts() function.
 * @param[in] fmt string format
 * @return number of printed characters
 */
#ifdef LIB_PRINTF
extern int pp_printf(const char *fmt, ...)
        __attribute__((format(printf,1,2)));
#else
static inline int pp_printf(const char *fmt, ...)
{
        return 0;
}
#endif


/**
 * It creates a new string according to the given format
 * @param[out] s output string
 * @param[in] fmt string format
 * @return number of printed characters
 */
#ifdef LIB_PRINTF
extern int pp_sprintf(char *s, const char *fmt, ...)
        __attribute__((format(printf,2,3)));
#else
static inline int pp_sprintf(char *s, const char *fmt, ...)
{
        return 0;
}
#endif


/**
 * It prints a string on the serial interface.
 * Internally, it uses the puts() function.
 * @param[in] fmt string format
 * @param[in] args list of arguments according to the string format
 * @return number of printed characters
 */
#ifdef LIB_PRINTF
extern int pp_vprintf(const char *fmt, va_list args);
#else
static inline int pp_vprintf(const char *fmt, va_list args)
{
        return 0;
}
#endif


/**
 * It creates a new string according to the given format
 * @param[out] s output string
 * @param[in] fmt string format
 * @param[in] args list of arguments according to the string format
 * @return number of printed characters
 */
#ifdef LIB_PRINTF
extern int pp_vsprintf(char *buf, const char *, va_list args)
        __attribute__ ((format (printf, 2, 0)));
#else
static inline int pp_vsprintf(char *buf, const char *fmt, va_list args)
{
        return 0;
}
#endif

/**@}*/

#endif
